"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _require = require("./helpers"),
    existingMagicCommentChunkName = _require.existingMagicCommentChunkName,
    getMagicCommentChunkName = _require.getMagicCommentChunkName,
    getImportArgPath = _require.getImportArgPath,
    addChunkNameToNode = _require.addChunkNameToNode;

var asyncComponentImportNames = [];
var validImportSources = ["@jaredpalmer/after", "jaredpalmer/after/asyncComponent"];

function _default(_ref) {
  var t = _ref.types;
  return {
    name: "after-async-component",
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (t.isStringLiteral(path.node.source) && validImportSources.includes(path.node.source.value)) {
          var specifiers = path.node.specifiers;
          var asyncComponentImport = specifiers.filter(function (specifier) {
            return t.isImportDefaultSpecifier(specifier) || t.isImportSpecifier(specifier) && specifier.imported.name === "asyncComponent";
          });
          asyncComponentImport.forEach(function (asyncComponentImport) {
            asyncComponentImportNames.push(asyncComponentImport.local.name);
          });
        }
      },
      CallExpression: function CallExpression(path, _ref2) {
        var opts = _ref2.opts;

        if (asyncComponentImportNames.length >= 1, // check if the function that called is named "asyncComponent" or named export { asyncComponent as foo }
        t.isIdentifier(path.node.callee) && asyncComponentImportNames.includes(path.node.callee.name) && // check if function is property of an object
        t.isObjectProperty(path.parentPath) && // check if key of property is "component"
        t.isIdentifier(path.parentPath.node.key, {
          name: "component"
        })) {
          path.traverse(importVisitor, {
            parentPath: path.parentPath,
            prefix: opts.prefix,
            t: t
          });
        }
      }
    }
  };
}

var importVisitor = {
  Import: function Import(path) {
    var argPath = getImportArgPath(path);
    var node = argPath.node;
    var generatedChunkName = getMagicCommentChunkName(node);

    if (generatedChunkName === "[request]") {
      return;
    }

    var existingChunkName = existingMagicCommentChunkName(node);
    var chunkName = existingChunkName || convertChunkName(generatedChunkName, this.prefix);
    addChunkNameToNode(argPath, chunkName);
    this.parentPath.insertBefore(this.t.objectProperty(this.t.stringLiteral("chunkName"), this.t.stringLiteral(chunkName)));
  }
};

function convertChunkName(chunkName) {
  var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "AfterChunk-";
  return prefix + chunkName.replace("/", "-");
}